import 'dart:async';

import 'consentable_type.dart';
import 'package:flutter/services.dart';

/// [AppconsentTv] is the entry point to interract with our CMP.
///
/// You first configure an instance of it calling [AppconsentTv.setup].
/// Then use [presentNotice] to display the CMP UI if needed.
class AppconsentTv {
  static const MethodChannel _channel = MethodChannel('appconsent_tv');

  /// Setup the AppConsent CMP.
  ///
  /// Setup the CMP by calling this function, passing it your [appKey].
  /// The CMP detects if the app is running on a device that is inside a GDPR country, with [forceApplyGDPR] you can choose to bypass the check, and let the CMP always behave according to GDPR, if ignored, this defaults to `false`.
  /// On iOS, the ATT checks can be enforced with [forceATT], this option is ignored on Android.
  static Future<void> setup(
          String appKey, bool forceApplyGDPR, bool forceATT) async =>
      await _channel.invokeMethod("setup", {
        "appKey": appKey,
        "forceApplyGDPR": forceApplyGDPR,
        "forceATT": forceATT
      });

  /// Displays the CMP Notice if needed.
  ///
  /// Once the CMP has been setup with [setup], call this to display the Notice UI.
  /// Internally, this function checks if a user has already saved their preferences:
  ///  * if yes, then the CMP Notice is not displayed.
  ///  * if not, the CMP Notice is displayed, so the user can save their preferences.
  ///
  /// The [force] option tells the Notice to always display the preferences screen, even if the user has already saved their preferences. The default behavior is `false`.
  /// If set to `true`, then the bahavior changes has such:
  ///  * if user has already saved their preferences before, the introduction and global choices screen is not shown, and the CMP Notice displays the detailed preferences page. This is used to allow the user to change their preferences.
  ///  * if not, the CMP Notice is displayed as usual.
  ///
  /// This function returns as soon as the CMP Notice has been displayed, so when it returns, the user may not have saved their preferences yet.
  static Future<void> presentNotice(bool force) async {
    await _channel.invokeMethod("presentNotice", {"force": force});
  }

  /// Clears user consents preferences.
  ///
  /// This only clears local consent in SharedPreferences, all of the IABTCF SharedPreferences are erased.
  static Future<void> clearConsent() async {
    await _channel.invokeMethod('clearConsent');
  }

  /// Returns `true`if an update to the Notice and consents is needed.
  static Future<bool> get checkForUpdate async {
    final bool result = await _channel.invokeMethod('checkForUpdate');
    return result;
  }

  /// Returns `true` if user is subject to GDPR. It depends on forceApplyGDPR and phone's locale.
  static Future<bool> get isSubjectToGDPR async {
    final bool result = await _channel.invokeMethod('isSubjectToGDPR');
    return result;
  }

  /// Returns `true` if user has already saved their preferences.
  static Future<bool> get consentGiven async {
    final bool result = await _channel.invokeMethod('consentGiven');
    return result;
  }

  /// Returns `true` if user gave consent to Geolocation purpose.
  static Future<bool> get geolocationConsentGiven async {
    final bool result = await _channel.invokeMethod('geolocationConsentGiven');
    return result;
  }

  /// Returns `true` if consentable of [ConsentableType] [type] with ID [ìabId] is ALLOWED.
  static Future<bool> consentableAllowed(
      int iabId, ConsentableType type) async {
    final bool result = await _channel.invokeMethod(
        'consentableAllowed', {"iabId": iabId, "consentableType": type.index});
    return result;
  }
}
